import numpy as np
import pandas as pd
from logs import setup_logging

logger = setup_logging()


def sto06(Hs0, Tp, beta, **kwargs):
    """
    :param Hs0: List or float of offshore significant wave height values
    :param Tp: List or float of peak wave period
    :param beta: List of float of beach slope
    :return: Float or list of R2, setup, S_total, S_inc and S_ig values
    """

    df = pd.DataFrame(
        {"Hs0": Hs0, "Tp": Tp, "beta": beta}, index=[x for x in range(0, np.size(Hs0))]
    )

    df["Lp"] = 9.8 * df["Tp"] ** 2 / 2 / np.pi

    # General equation
    df["S_ig"] = pd.to_numeric(0.06 * np.sqrt(df["Hs0"] * df["Lp"]), errors="coerce")
    df["S_inc"] = pd.to_numeric(
        0.75 * df["beta"] * np.sqrt(df["Hs0"] * df["Lp"]), errors="coerce"
    )
    df["setup"] = pd.to_numeric(
        0.35 * df["beta"] * np.sqrt(df["Hs0"] * df["Lp"]), errors="coerce"
    )
    df["S_total"] = np.sqrt(df["S_inc"] ** 2 + df["S_ig"] ** 2)
    df["R2"] = 1.1 * (df["setup"] + df["S_total"] / 2)

    # Dissipative conditions
    dissipative = df["beta"] / (df["Hs0"] / df["Lp"]) ** (0.5) <= 0.3

    df.loc[dissipative, "setup"] = 0.016 * (df["Hs0"] * df["Lp"]) ** (0.5)  # eqn 16
    df.loc[dissipative, "S_total"] = 0.046 * (df["Hs0"] * df["Lp"]) ** (0.5)  # eqn 17
    df.loc[dissipative, "R2"] = 0.043 * (df["Hs0"] * df["Lp"]) ** (0.5)  # eqn 18

    return (
        float_or_list(df["R2"].tolist()),
        float_or_list(df["setup"].tolist()),
        float_or_list(df["S_total"].tolist()),
        float_or_list(df["S_inc"].tolist()),
        float_or_list(df["S_ig"].tolist()),
    )


def hol86(Hs0, Tp, beta, **kwargs):
    df = pd.DataFrame(
        {"Hs0": Hs0, "Tp": Tp, "beta": beta}, index=[x for x in range(0, np.size(Hs0))]
    )

    df["Lp"] = 9.8 * df["Tp"] ** 2 / 2 / np.pi

    df["setup"] = 0.2 * df["Hs0"]
    df["R2"] = 0.83 * df["beta"] * np.sqrt(df["Hs0"] * df["Lp"]) + df["setup"]

    df["S_ig"] = np.nan
    df["S_inc"] = np.nan
    df["S_total"] = np.nan

    return (
        float_or_list(df["R2"].tolist()),
        float_or_list(df["setup"].tolist()),
        float_or_list(df["S_total"].tolist()),
        float_or_list(df["S_inc"].tolist()),
        float_or_list(df["S_ig"].tolist()),
    )


def nie91(Hs0, Tp, beta, **kwargs):
    df = pd.DataFrame(
        {"Hs0": Hs0, "Tp": Tp, "beta": beta}, index=[x for x in range(0, np.size(Hs0))]
    )

    df["Lp"] = 9.8 * df["Tp"] ** 2 / 2 / np.pi
    df["Ls"] = df["Lp"]  # Need to make this approximation, refer to Atkinson 2017
    df["Hrms"] = df["Hs0"] / np.sqrt(
        2
    )  # Acceptable approximation, refer to Atkinson 2017

    df.loc[df.beta >= 0.1, "LR"] = 0.6 * df["beta"] * np.sqrt(df["Hrms"] * df["Ls"])
    df.loc[df.beta < 0.1, "LR"] = 0.06 * np.sqrt(df["Hrms"] * df["Ls"])

    df["R2"] = 1.98 * df["LR"]

    # Note that this should be the level above Z100%, which in this case is taken as the time varying tide level,
    # even though minimum run-down can still occur below tide SWL.

    df["setup"] = np.nan
    df["S_ig"] = np.nan
    df["S_inc"] = np.nan
    df["S_total"] = np.nan

    return (
        float_or_list(df["R2"].tolist()),
        float_or_list(df["setup"].tolist()),
        float_or_list(df["S_total"].tolist()),
        float_or_list(df["S_inc"].tolist()),
        float_or_list(df["S_ig"].tolist()),
    )


def atk18(Hs0, Tp, beta):
    pass


def pow18(Hs0, Tp, beta, r, **kwargs):
    logger.info("Calculating runup using Power et al. (2018)")
    df = pd.DataFrame(
        {"Hs0": Hs0, "Tp": Tp, "beta": beta, "r": r},
        index=[x for x in range(0, np.size(Hs0))],
    )

    df["Lp"] = 9.8 * df["Tp"] ** 2 / 2 / np.pi
    df["x1"] = df["Hs0"] / df["Lp"]
    df["x2"] = df["beta"]
    df["x3"] = df["r"] / df["Hs0"]

    df["R2"] = df.Hs0 * (
        (df.x2 + (((df.x3 * 3) / np.exp(-5)) * ((3 * df.x3) * df.x3)))
        + ((((df.x1 + df.x3) - 2) - (df.x3 - df.x2)) + ((df.x2 - df.x1) - df.x3))
        + (((df.x3 ** df.x1) - (df.x3 ** (1 / 3))) - (np.exp(df.x2) ** (df.x1 * 3)))
        + np.sqrt((((df.x3 + df.x1) - df.x2) - (df.x2 + np.log10(df.x3))))
        + ((((df.x2 ** 2) / (df.x1 ** (1 / 3))) ** (df.x1 ** (1 / 3))) - np.sqrt(df.x3))
        + (
            (df.x2 + ((df.x3 / df.x1) ** (1 / 3)))
            + (np.log(2) - (1 / (1 + np.exp(-(df.x2 + df.x3)))))
        )
        + ((np.sqrt(df.x3) - (((3 ** 2) + 3) * (df.x2 ** 2))) ** 2)
        + ((((df.x3 * -5) ** 2) ** 2) + (((df.x3 + df.x3) * df.x1) / (df.x2 ** 2)))
        + np.log(
            (np.sqrt(((df.x2 ** 2) + (df.x3 ** (1 / 3)))) + ((df.x2 + 3) ** (1 / 3)))
        )
        + (
            (((df.x1 / df.x3) * (-5 ** 2)) * (df.x3 ** 2))
            - np.log10((1 / (1 + np.exp(-(df.x2 + df.x3)))))
        )
        + (df.x1 ** df.x3)
        + np.exp(-((((df.x3 / df.x1) ** np.exp(4)) + (np.exp(df.x3) ** 3)) ** 2))
        + np.exp((np.log((df.x2 - df.x3)) - np.log(np.exp(-((-1 + df.x1) ** 2)))))
        + ((np.sqrt(4) * (((df.x3 / df.x2) - df.x2) - (0 - df.x1))) ** 2)
        + (2 * ((((-5 * df.x3) + df.x1) * (2 - df.x3)) - 2))
        + ((np.sqrt(4) * (((df.x3 / df.x2) - df.x2) - (0 - df.x1))) ** 2)
        + ((((-5 + df.x1) - df.x2) * (df.x2 - df.x3)) * ((df.x1 - df.x2) - (-4 ** -5)))
        + (np.exp(-((df.x2 + (-5 - df.x1)) ** 2)) + ((df.x2 + 5) * (df.x3 ** 2)))
        + np.sqrt(
            1
            / (
                1
                + np.exp(
                    -(
                        (np.exp(df.x1) - np.exp(-((df.x3 + df.x3) ** 2)))
                        + ((df.x1 ** df.x3) - (df.x3 * 4))
                    )
                )
            )
        )
        + (
            (
                np.exp(
                    -(
                        (
                            (
                                (
                                    np.exp(
                                        -(
                                            (
                                                (np.sqrt(df.x3) * 4)
                                                + (1 / (1 + np.exp(-(df.x2 + 2))))
                                            )
                                            ** 2
                                        )
                                    )
                                )
                                ** 2
                            )
                            + df.x1
                        )
                        ** 2
                    )
                )
            )
            ** 3
        )
    )

    df["setup"] = np.nan
    df["S_ig"] = np.nan
    df["S_inc"] = np.nan
    df["S_total"] = np.nan

    return (
        float_or_list(df["R2"].tolist()),
        float_or_list(df["setup"].tolist()),
        float_or_list(df["S_total"].tolist()),
        float_or_list(df["S_inc"].tolist()),
        float_or_list(df["S_ig"].tolist()),
    )


def beu(Hs0, Tp, beta):
    pass


def float_or_list(a):
    """
    If only one value in the array, return the float, else return a list
    :param a:
    :return:
    """
    if len(a) == 1:
        return a[0]
    else:
        return list(a)


if __name__ == "__main__":
    pass
